###############################################################################
#                 Undernet Channel Service Management Script                  #
###############################################################################
# UndernetCS-Auth.tcl                                                         #
###############################################################################
# (c) 2001 RICHARD Yann alias Ze <Ze@administrateur.net>                      #
###############################################################################

###############################################################################
# This program is distributed under the GNU General Public Licence            #
#  See the LICENCE file for more information.                                 #
###############################################################################

# UndernetCS:Auth:IsOkX
#   Return: 1 if  o The script is actived  AND
#                 o The script is enabled for 'channel'
#                 o Eggdrop have an access to X for 'channel' AND
#                 o nick = X and 'userhost' is the same that X's user@host,
#           0 otherwise.
proc UndernetCS:Auth:IsOkX {nick userhost channel debugaction} {
    
    global UndernetCS_Prompt
    # Testing if the script is actived.
    if { ![UndernetCS:IsActived] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. Script is actualy disabled."
        }
        return 0
    }

    # Testing if script is enabled for channel
    if { ![UndernetCS:IsEnabled $channel] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. $channel is actualy disabled."
        }
        return 0
    }

    # Testing if eggdrop have an access for this channel.
    if { ![UndernetCS:HaveAccess $channel] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. No access for this channel."
        }
        return 0
    }
    # Verify if it's the real X :))
    # In theory the bind with $UndernetCS_XMask do this verification ...
    if { ![UndernetCS:IsX "$nick" "$userhost"] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. (Official Channel Service ?)."
        }
        return 0
    }
    if { [UndernetCS:Auth:DebugIsOn] } {
        putlog "$UndernetCS_Prompt ${debugaction}."
    }
    return 1
}

# UndernetCS:Auth:IsOkEgg
#   Return: 1 if o 'nick' is the eggdrop AND
#                o The script is actived AND 
#                o if eggdrop have an access to X for 'channel' AND
#                o if X is on 'channel',
#           0 otherwise.
proc UndernetCS:Auth:IsOkEgg  {nick userhost channel debugaction} {
    
    global UndernetCS_Prompt botnick UndernetCS_Nick

    # first verify if it's the eggie
    if { $botnick != $nick } {
        return 0
    }
    # Testing if the script is actived.
    if { ![UndernetCS:IsActived] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. Script is actualy disabled."
        }
        return 0
    }

    # Testing if eggdrop have an access for this channel.
    if { ![UndernetCS:HaveAccess $channel] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. No access for this channel."
        }
        return 0
    }

    # Is X are on the channel ?
    if { ![onchan "$UndernetCS_Nick" "$channel"] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. X are not in $channel."
        }
        return 0
    }

    # Is eggdrop access is suspended ?
    if { [UndernetCS:IsSuspended $channel] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. (Access is suspended for $channel)."
        }
        return 0
    }
    
    if { [UndernetCS:Auth:DebugIsOn] } {
        putlog "$UndernetCS_Prompt ${debugaction}."
    }
    return 1
}

# -----------------------------------------------------------------------------
# ----------------------------------------------------------- EVENTS ----------
# -----------------------------------------------------------------------------

bind join - "* $UndernetCS_XMask" UndernetCS:Auth:XJoin
proc UndernetCS:Auth:XJoin {nick userhost handle channel} {
    
    set debugaction "${nick}(${userhost}) join $channel."    

    # Verify if nick is X, if script enabled and if eggie
    # have an access for this channel.
    if { ![UndernetCS:Auth:IsOkX "$nick" "$userhost" "$channel" "$debugaction"] } {
        return
    }
    if { [UndernetCS:IsSuspended $channel] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. (Access is suspended for $channel)."
        }
        return 0
    }
    # It is possible that X rejoin after a request from 450+... and eggdrop is auth.
    if { [UndernetCS:IsAuth] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. We are already Authenticated."
        }
        return 0
    }
    UndernetCS:Auth:Login2X 
}

bind REJN - "* $UndernetCS_XMask" UndernetCS:Auth:XSplitRejoin
proc UndernetCS:Auth:XSplitRejoin {nick userhost handle channel} {
    set debugaction "${nick}(${userhost}) join (After a split) $channel."    

    # Verify if nick is X, if script enabled and if eggie
    # have an access for this channel.
    if { ![UndernetCS:Auth:IsOkX "$nick" "$userhost" "$channel" "$debugaction"] } {
        return
    }
    if { [UndernetCS:IsSuspended $channel] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. (Access is suspended for $channel)."
        }
        return 0
    }
    UndernetCS:Auth:Login2X 
}

bind part - "* $UndernetCS_XMask" UndernetCS:Auth:XPart
proc UndernetCS:Auth:XPart {nick userhost handle channel {msg ""}} {
    set debugaction "${nick}(${userhost}) has left $channel."
    
    # Verify if nick is X, if script enabled and if eggie
    # have an access for this channel.    
    if { ![UndernetCS:Auth:IsOkX "$nick" "$userhost" "$channel" "$debugaction"] } {
        return
    }
    # We verify if X is already present in other channel and we consider
    # if X not present that eggdrop aren't authenticated now.
    foreach chan [channels] {
        if { $chan == $channel } {
            continue
        }
        if { [onchan $nick $chan] } {
            return
        }
    }
    # X isn't on others channels..
    # .. so we are not auth now ..
    UndernetCS:SetAuthStatus 0
}

bind SIGN - "* $UndernetCS_XMask" UndernetCS:Auth:XQuit
proc UndernetCS:Auth:XQuit {nick userhost handle channel reason} {
    set debugaction "${nick}(${userhost}) has quit IRC $channel."
    
    # Verify if nick is X, if script enabled and if eggie
    # have an access for this channel.    
    if { ![UndernetCS:Auth:IsOkX "$nick" "$userhost" "$channel" "$debugaction"] } {
        return
    }
    # X quit IRC .. so we are not auth now ..
    UndernetCS:SetAuthStatus 0
}

bind SPLT - "* $UndernetCS_XMask" UndernetCS:Auth:XSplit
proc UndernetCS:Auth:XSplit {nick userhost handle channel} {
   set debugaction "${nick}(${userhost}) has gets netsplit $channel."
    
    # Verify if nick is X, if script enabled and if eggie
    # have an access for this channel.    
    if { ![UndernetCS:Auth:IsOkX "$nick" "$userhost" "$channel" "$debugaction"] } {
        return
    }
    # X quit IRC .. so we are not auth now ..
    UndernetCS:SetAuthStatus 0
}

bind evnt - init-server UndernetCS:Auth:ServerConnect
proc UndernetCS:Auth:ServerConnect {type} {
    global UndernetCS_Prompt 
    if { [UndernetCS:Auth:DebugIsOn] } {
        putlog "$UndernetCS_Prompt I'am just connected to an IRC server..."
    }
    # Send a login request now without know if X is on IRC...
    UndernetCS:Auth:Login2X
}

bind evnt - disconnect-server UndernetCS:Auth:ServerDisconnect
proc UndernetCS:Auth:ServerDisconnect {type} {
    global UndernetCS_Prompt 
    # We are disconnected.. so we are not authenticated now ..
    if { [UndernetCS:Auth:DebugIsOn] } {
        putlog "$UndernetCS_Prompt I'am just disconnected from IRC."
    }
    UndernetCS:SetAuthStatus 0
}

bind part - * UndernetCS:Auth:EggPart
proc UndernetCS:Auth:EggPart {nick userhost handle channel msg} {
    global UndernetCS_Prompt UndernetCS_Nick

    set debugaction "${nick}(${userhost}) has left $channel. <-- It's ME?"

    if { ![UndernetCS:Auth:IsOkEgg "$nick" "$userhost" "$channel" "$debugaction"] } {
        return
    }
    if { ![validchan $channel] } {
        # This chan is just remove from eggdrop ..
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt $debugaction ; This chan isn't monitored, remove it from UndernetCS"
        }
        UndernetCS:RemChan $channel
    } else {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt $debugaction ; this chan is probably +inactive"
        }
        foreach chan [channels] {
            if { $chan == $channel } {
                continue
            }
            if { [onchan $UndernetCS_Nick $chan] } {
                if { [UndernetCS:Auth:DebugIsOn] } {
                    putlog "$UndernetCS_Prompt $debugaction ; X is present on $chan so we are already authenticated."
                }
                return
            }
        }
        # X isn't on others channels..
        # .. so we are not auth now ..
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt $debugaction ; X isn't present on others channel, we are unauthenticated."
        }
        UndernetCS:SetAuthStatus 0
    }
}

bind join - * UndernetCS:Auth:EggJoinTimer
proc UndernetCS:Auth:EggJoinTimer {nick userhost handle channel} {
    global UndernetCS_ChannelSetting UndernetCS_Prompt

    if { [info exists UndernetCS_ChannelSetting([string toupper $channel],LEVEL)] } {
        # Parsing nick userhost handle and channel for [] and {} to add \...
        set nick [UndernetCS:Parse $nick]
        set handle [UndernetCS:Parse $handle]
        set userhost [UndernetCS:Parse $userhost]
        set channel [UndernetCS:Parse $channel]
        set cmd "UndernetCS:Auth:EggJoin $nick $userhost $handle $channel"
        set t [utimer 5 $cmd]
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt EggJoinTimer --> $t"
        }
    } else {
        # This is a new channel !
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt New channel detected: $channel"
        }
        UndernetCS:AddChan "$channel"
    }
}

proc UndernetCS:Auth:EggJoin {nick userhost handle channel} {
    global UndernetCS_Prompt

    set debugaction "${nick}(${userhost}) join $channel. <-- It's ME?"

    if { ![UndernetCS:Auth:IsOkEgg "$nick" "$userhost" "$channel" "$debugaction"] } {
        return
    }

    # Verify if eggdrop isn't already authenticated
    if { [UndernetCS:IsAuth] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt I'am already authenticated."
        }
        # Eggdrop is so curently authenticated...
        # Verify if suspended ...
        if { [UndernetCS:IsSuspended $channel] } {
            if { [UndernetCS:Auth:DebugIsOn] } {
                putlog "$UndernetCS_Prompt My access is suspended on $channel."
            }
            return
        }
        # Verify if AUTOMODE is on OP and if not sending op command to X
        if { ![UndernetCS:IsAutoModeOp $channel] } {
            # Sending op command to X
            set res [UndernetCS:Cmd:Op $channel]
            # TODO use the result ....
        }
        return
    }
    # Verify if eggdrop is waiting for a reply to a login request.
    if { [UndernetCS:IsWaitingForReply] } {
        # TODO verify a timeout of the request ...
        return
    }
    # We aren't authenticated and eggdrop isn't waiting for a reply from X.
    # so we are going to start the auth procedure.
    UndernetCS:Auth:Login2X
    # We don't send now a op cmd, this events is managed when auth succ
    # is receive.
}

proc UndernetCS:Auth:IsOkX_Notice {nick userhost dest debugaction} {
    global botnick UndernetCS_Prompt 
    # Testing if dest is eggdrop.
    if { $dest != $botnick } {
        return
    }

    # Verify if the script is active
    if { ![UndernetCS:IsActived] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. Script is actualy disabled."
        }
        return 0
    }

    # Verify if it's the real X :))
    if { ![UndernetCS:IsX "$nick" "$userhost"] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt ${debugaction}. (Official Channel Service ?)."
        }
        return 0
    }
    if { [UndernetCS:Auth:DebugIsOn] } {
        putlog "$UndernetCS_Prompt ${debugaction}."
    }
    return 1
}

bind notc - "AUTHENTICATION SUCCESSFUL as*" UndernetCS:Auth:XNotice_AuthSucc
proc UndernetCS:Auth:XNotice_AuthSucc {nick userhost handle text dest} {
    global UndernetCS_Prompt 
    set debugaction "Received notice from ${nick}(${userhost}): $text (auth succ)"
    if { ![UndernetCS:Auth:IsOkX_Notice "$nick" "$userhost" "$dest" "$debugaction"] } {
        return
    }
    set textnotice [string toupper $text]
    
    # We receive a notice from X who said, that auth is succ, paranoia mode:
    # we verify if username is the one that are saved in this script :)
    if { [string tolower [lindex $textnotice 3]] != [string tolower [UndernetCS:GetUserId]] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            set alert "!! I want this :Auth succ as [UndernetCS:GetUserId] !!"
            putlog "$UndernetCS_Prompt ${debugaction}. $alert"
        }
        UndernetCS:SetAuthStatus 0
        UndernetCS:Auth:Login2X
        return
    }
    UndernetCS:SetAuthStatus 1
}

bind notc - "Sorry, You are already authenticated as*" UndernetCS:Auth:XNotice_AlreadyAuth
proc UndernetCS:Auth:XNotice_AlreadyAuth {nick userhost handle text dest} {
    global UndernetCS_Prompt     
    set debugaction "Received notice from ${nick}(${userhost}): $text (already auth)"
    if { ![UndernetCS:Auth:IsOkX_Notice "$nick" "$userhost" "$dest" "$debugaction"] } {
        return
    }
    set textnotice [string toupper $text]
    
    # We receive a notice from X who said, we are already auth, paranoia mode:
    # we verify if username is the one that are saved in this script :)
    if { [string tolower [lindex $textnotice 6]] != [string tolower [UndernetCS:GetUserId]] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            set alert "!! I want this : as [UndernetCS:GetUserId] !!"
            putlog "$UndernetCS_Prompt ${debugaction}. $alert"
        }
        UndernetCS:Auth:Login2X 
        return
    }
    # We are auth ..
    UndernetCS:SetAuthStatus 1
}

bind notc - "Sorry, You must be logged in to use this command*" UndernetCS:Auth:XNotice_NotLogged
proc UndernetCS:Auth:XNotice_NotLogged {nick userhost handle text dest} {
    global UndernetCS_Prompt 
    set debugaction "Received notice from ${nick}(${userhost}): $text (not logged)"
    if { ![UndernetCS:Auth:IsOkX_Notice "$nick" "$userhost" "$dest" "$debugaction"] } {
        return
    }
    set textnotice [string toupper $text]
    # We are not auth ..
    UndernetCS:SetAuthStatus 0
    # Sending login command ..
    UndernetCS:Auth:Login2X 
}

bind notc - "AUTHENTICATION FAILED as*" UndernetCS:Auth:XNotice_AuthFailed
proc UndernetCS:Auth:XNotice_AuthFailed {nick userhost handle text dest} {
    global UndernetCS_Prompt 
    set debugaction "Received notice from ${nick}(${userhost}): $text (auth failed)"
    if { ![UndernetCS:Auth:IsOkX_Notice "$nick" "$userhost" "$dest" "$debugaction"] } {
        return
    }
    set textnotice [string toupper $text]
    # We receive a notice from X who said, auth failed as ***, paranoia mode:
    # we verify if username is the one that are saved in this script :)
    if { [string tolower [lindex $textnotice 3]] != [string tolower [UndernetCS:GetUserId]] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            set alert "!! I want this : as [UndernetCS:GetUserId] !!"
            putlog "$UndernetCS_Prompt ${debugaction}. $alert"
        }
        # Send another
        UndernetCS:Auth:Login2X
        return
    }
    # Auth failed ...
    UndernetCS:SetAuthStatus -1
}

bind notc - "Your access on*has been suspended." UndernetCS:Auth:XNotice_Suspended
proc UndernetCS:Auth:XNotice_Suspended {nick userhost handle text dest} {
    global UndernetCS_Prompt 
    set debugaction "Received notice from ${nick}(${userhost}): $text (Access suspended)"
    if { ![UndernetCS:Auth:IsOkX_Notice "$nick" "$userhost" "$dest" "$debugaction"] } {
        return
    }
    set textnotice [string toupper $text]
    set channel [lindex $textnotice 3]
    # set that access for this channel is suspended..
    UndernetCS:SetSuspended $channel 1
}

bind mode - * UndernetCS:Auth:OnEggDeOp
proc UndernetCS:Auth:OnEggDeOp {nick userhost handle channel modech victim} {
    global botnick UndernetCS_Prompt

    if { $modech != "-o" } { return 1 }

    if { [string tolower $victim] == [string tolower $botnick] } {
    	# Eggdrop was just deopped by $nick..
    	if { [matchattr $handle m|] == 0 } {
    	    # This user is not authorised to do that ...
	    putlog "#$handle# $nick deop me on $channel."
    	    set res [UndernetCS:EggdropCanSendCommand2X "$channel"]
    	    if { $res == 1 } {
		UndernetCS:Cmd:Op "$channel"
		UndernetCS:Cmd:DeOp "$channel" "$nick"
		if { $handle == "*" } {
		    # This is an unknow user.
		    if { [UndernetCS:Auth:DebugIsOn] } {
			putlog "$UndernetCS_Prompt $nick deop me on $channel (User is unknow) send request to ban level 25 for 1 day."
		    }
		    UndernetCS:Cmd:Ban "$channel" "$nick" "25" "Deoping me is forbiden." "24"
		}
	    } else {
		# Testing why eggdrop can't send a command..
		# We do nothing (need-op analyze this)
		putlog "Hey! $nick ($handle) deop me on $channel"
	    }
    	} else {
	    # User are authorised.
	    if { [UndernetCS:Auth:DebugIsOn] } {
		putlog "$UndernetCS_Prompt $handle deop me on $channel"
	    }
    	    return 1
    	}
    }
    return 1
}

# ------------------------------------------------------ End of events proc ---
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# ---------------------------------------------------- Timer Management--------
# -----------------------------------------------------------------------------
proc UndernetCS:Auth:AuthTimeChecker {min hour day month year} {
    global UndernetCS_Prompt 
    if { [UndernetCS:IsAuth] } {
        # We are already authenticated...
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt Authentication checker: We are already Authenticated."
        }
        return 0
    }

    set timerlastsend [UndernetCS:IsWaitingForReply]
    if { $timerlastsend == 0 } {
        # this proc verify if last X's reply is "failed"
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt Authentication checker: We are not auth and not waiting for a reply."
        }
        UndernetCS:Auth:Login2X
        return 1
    }
    # Calculating if the last request is recent..
    # if more than 2 minutes we resend a login request ...
    set dif [expr $timerlastsend - [unixtime]]
    if { $dif > 119 } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt Authentication checker: Waiting for a reply since more than 2 min.. Sending a new one."
        }
        UndernetCS:Cmd:Login
        return 1
    }
    return 0
}

# -----------------------------------------------------------------------------
# --------------------------------------------------------- Sub Proc ----------
# -----------------------------------------------------------------------------
# UndernetCS:Auth:Login2X 
#   Send an command to authenticate the eggdrop to X.
#   the command is not send, if we have already send this command and
#   eggdrop waiting for a reply, and if the last login request is FAILED.
#   WARNING: in this command we don't verify if eggdrop is already auth to X
#   Return: 1 if command if send
#           0 if not.
proc UndernetCS:Auth:Login2X {} {
    global UndernetCS_Prompt UndernetCS_Nick
    # Verify if we are waiting for a reply to an auth command
    if { [UndernetCS:IsWaitingForReply] } {
        # Verifier le timeout ?
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt An authentication procedure is under processing."
        }
        return 0
    }
    
    # Verify if last anwser to auth request was "FAILED".
    if { [UndernetCS:IsAuthFailed] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt The last try to auth failed. Script is so disabled."
        }
        return 0
    }
    # Verify if X is at least on one channel, so verify if he is online.
    # If not on line we don't send a login request. In the future we will
    # use the ISON irc command to know if X is on IRC.
    set ison 0
    foreach chan [channels] {
        if { [onchan $UndernetCS_Nick $chan] } {
            set ison 1
        }
    }
    if { $ison == 1 } {
        set res [UndernetCS:Cmd:Login]
        return $res
    } else {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt X isn't on any channel who are monitored. So we don't send a login request."
        }
    }
    return 0
}

bind raw - 401 UndernetCS:Auth:RAW401
proc UndernetCS:Auth:RAW401 {from keyword args} {
    global UndernetCS_Prompt UndernetCS_Nick
    set args [split $args]
    set nick [string tolower [string index [lindex $args 1] 0]]
    if {  [string tolower $nick] != [string tolower $UndernetCS_Nick] } {
        if { [UndernetCS:Auth:DebugIsOn] } {
            putlog "$UndernetCS_Prompt RAW 401: X is not on IRC."
        }
        UndernetCS:SetAuthStatus 0
    } elseif { [UndernetCS:Auth:DebugIsOn] } {
        putlog "$UndernetCS_Prompt Received RAW 401: $args ($nick)"
    }
    return 1
}

# UndernetCS:Auth:DebugIsOn
#   Check if debug is enabled for the 'AUTH' section.
#   Return: 1 if enabled.
#           0 if disabled.
proc UndernetCS:Auth:DebugIsOn {} {
    global UndernetCS_Debug
    return $UndernetCS_Debug(AUTH)
}
### EOF ###