###############################################################################
#                 Undernet Channel Service Management Script                  #
###############################################################################
# UndernetCS-DCC.tcl                                                          #
###############################################################################
# Copyright (c) 2001 RICHARD Yann alias Ze <Ze@administrateur.net>            #
###############################################################################

###############################################################################
# This program is distributed under the GNU General Public Licence            #
#  See the LICENCE file for more information.                                 #
###############################################################################

####
# Tested: OK (17/03/2001)
bind dcc m|n -cs UndernetCS:DCC:UnactiveChan
proc UndernetCS:DCC:UnactiveChan {handle idx arg} {
   
    set usage "Usage: .-cs \[#channel\]"
    if { [llength $arg] == 0 } {
        set chan [string toupper [lindex [console $idx] 0]]
    } elseif { [llength $arg] == 1 } {
        set chan [string toupper $arg]
        # Verify if it's a valid channel
        if { ![validchan $chan] } {
            putdcc $idx "$chan isn't a valid channel."
            putdcc $idx "$usage"
            putcmdlog "#$handle# .+cs $arg (Bad # of arg)."
        }
    } else {
        putdcc $idx "$usage"
        putcmdlog "#$handle# .+cs $arg (Bad # of arg)."
        return 0
    }
    # Verify access flag ..
    if { ! ( [matchattr $handle n] || [matchattr $handle |m $chan] ) } {
        putdcc $idx "Sorry, you have insufficient access to perform that command."
        putcmdlog "#$handle# .+cs $arg (insufficient access)."
        return 0
    }
    
    if { ![UndernetCS:IsEnabled $chan] } {
        putdcc $idx "Status of $chan is already disabled."
        return 1
    }
    # Now unactive this chan if level is more than 0 ...
    UndernetCS:SetDisabled "$chan"
    UndernetCS:IO:SaveChannelSetting $chan
    putdcc $idx "Undernet Channel Service:"
    putdcc $idx "    Status for $chan is now: Disabled."
    return 1
}

####
# Tested: OK (17/03/2001)
bind dcc n| cs UndernetCS:DCC:CS
proc UndernetCS:DCC:CS {handle idx arg} {

    set usage "\n .cs set \[user <username>\] \[pass <password>\]"
    set usage "$usage \[check_auth <min>\] \[status <ON|OFF>\]\n"
    set usage "$usage .cs chanset \[#chan\] \[status <ON|OFF>\] \[automode <OP|VOICE|NONE>\]"
    set usage "$usage \[level <access level>\] \[check_susp <min>\]"

    if { [llength $arg] == 0 } {
        putdcc $idx "Usage: $usage"
        putcmdlog "#$handle# .cs $arg (Bad # of arg)."
        return 0
    }

    set cmd "[string tolower [lindex $arg 0]]"
    if { $cmd == "set" } {
        # Verify access flag ..
        if { ![matchattr $handle n] } {
            putdcc $idx "Sorry, you have insufficient access to perform that command."
            putcmdlog "#$handle# .cs $arg (insufficient access)."
            return 0
        }
        set res [UndernetCS:DCC:CS_Set $handle $idx "[lrange $arg 1 end]"]
        return $res
    }

    if { $cmd == "chanset" } {
        set argline [string tolower [lrange $arg 1 end]]
        set res [UndernetCS:DCC:CS_ChanSet $handle $idx $argline]
        return $res
    }
    if { $cmd == "list" } {
        putdcc $idx "Undernet Channel Service:"
        putdcc $idx "    Global setting:"
        putdcc $idx "        Username: [UndernetCS:GetUserId]"
        putdcc $idx "        Password: [UndernetCS:GetPass]"
        putdcc $idx "        Status: [UndernetCS:GetWordScriptStatus]"
        putdcc $idx "        Check Auth Frequency: Each [UndernetCS:GetAuthCheckFrequency] minutes"
        foreach channel [channels] {
            set chan [string toupper $channel]
            putdcc $idx "    $channel setting:"
            putdcc $idx "        Status : [UndernetCS:GetWordChannelStatus $chan]"
            putdcc $idx "        Level : [UndernetCS:GetLevel $chan]"
            putdcc $idx "        AutoMode : [UndernetCS:GetAutoMode $chan]"
            putdcc $idx "        Suspended : [UndernetCS:IsSuspended $chan]"
            putdcc $idx "        Check Auth when access suspended: Each [UndernetCS:GetSuspendedAuthCheckFrequency $chan] minutes"
        }
        return 1
    }
    putdcc $idx "$usage"
    putdcc $idx "Command must be set, chanset or list..."
    putcmdlog "#$handle# .cs $arg (bad first arg)."
    return 0
}

####
# Tested: OK (17/03/2001)
proc UndernetCS:DCC:CS_Set {handle idx arg} {

    set res "Channel Service:\n"
    set userset 0
    set passset 0
    set authchk 0
    set status 0

    if { [llength $arg] == 0 } {
        putdcc $idx "Undernet Channel Service:"
        putdcc $idx "    Status: [UndernetCS:GetWordScriptStatus]"
        putdcc $idx "    Bot's Username: [UndernetCS:GetUserId]"
        putdcc $idx "    Bot's Password: [UndernetCS:GetPass]"
        putdcc $idx "    Authentication is checked each [UndernetCS:GetAuthCheckFrequency] minutes."
        return 1
    }
    set i 0
    foreach {cmd val} $arg {
        set cmd [string tolower $cmd]
        if { ($cmd == "user") && ($userset == 0) } {
            set userset $val
        } elseif { ($cmd == "pass") && ($passset == 0) } {
            incr i 1
            set passset [lrange $arg $i end]
            break
        } elseif { ($cmd == "check_auth") && ($authchk == 0) } {
            set authchk $val
        } elseif { ($cmd == "status") && ($status == 0) } {
            set status [string toupper $val]
        } else {
            # Hummm
            set usage ".cs set \[user <username>\] \[check_auth <min>\]"
            set usage "$usage \[status <ON|OFF>\] \[pass <password>\]"
            putdcc $idx "$usage"
            putdcc $idx "'pass' MUST be the last element when you set several option."
            putcmdlog "#$handle# .cs $arg (!!Bad # of arg!!)."
            return 0
        }
        incr i 2
    }
    set displayarg ""
    set needsave 0
    if { $userset != 0 } {
        if { [string tolower [UndernetCS:GetUserId]] == [string tolower $userset] } {
            set res "$res    No change to do for eggdrop's username.\n"
        } else {
            UndernetCS:SetUserId $userset
            set res "$res    Internal value of X's Username changed to \"$userset\".\n"
            set needsave 1
        }
        set displayarg "user [UndernetCS:GetUserId]"
    }
    if { $passset != 0 } {
        if { $passset == [UndernetCS:GetPass] } {
            set res "$res    No change to do for X eggdrop's password.\n"
        } else {
            UndernetCS:SetPass "$passset"
            set res "$res    Internal value of X's Password changed.\n"
            set needsave 1
        }
        set displayarg "$displayarg pass (Something...)"
        if { [UndernetCS:IsAuthFailed] } {
            # Retry now to login to X.
            UndernetCS:SetAuthStatus 0
            UndernetCS:Auth:Login2X
        }
    }
    if { $authchk  != 0 } {
        if { $authchk  == [UndernetCS:GetAuthCheckFrequency] } {
            set res "$res    No change to do for Authentication check frequency.\n"
        } else {
            UndernetCS:SetAuthCheckFrequency $authchk
            set res "$res    Authentication is now check each $authchk minutes.\n"
            set needsave 1
        }
        set displayarg "$displayarg auth_check $authchk"
    }
    if { $status != 0 } {
        if { [UndernetCS:IsActived] == 1 } {
            set current "ON"
            set changeto 0 
        } else {
            set current "OFF"
            set changeto 1
        }
        if { $status == $current } {
            set res "$res    No change to do for status of the script.\n"
        } else {
            UndernetCS:SetGlobalStatus $changeto
            set res "$res    The script is now [UndernetCS:GetWordScriptStatus].\n"
            set needsave 1
        }
        set displayarg "$displayarg status $status"
    }
    putdcc $idx "$res"
    if { $needsave == 1 } {
        UndernetCS:IO:SaveGlobalSetting
    }
    putcmdlog "#$handle# .cs $displayarg."
    return 0
}

####
# Tested: OK (17/03/2001)
proc UndernetCS:DCC:CS_ChanSet {handle idx arg} {

    set usage ".cs chanset \[#chan\] \[status <ON|OFF>\] \[automode <OP|VOICE|NONE>\]"
    set usage "$usage \[level <access level>\] \[check_susp <min>\]"
    set chan ""
    
    if { [llength $arg] == 0 } {
        set chan [lindex [console $idx] 0]
    } elseif { [validchan [lindex $arg 0]] } {
    	set chan [lindex $arg 0]
    	set arg [lrange $arg 1 end]
    } else {
    	set chan [lindex [console $idx] 0]
    }

    # Verify access flag for this user/channel.
    if { ! ( ( [matchattr $handle n] == 1 ) || ( [mattchttr $handle |n $chan] == 1 ) ) } {
            putdcc $idx "Sorry, you have insufficient access to perform that command."
            putcmdlog "#$handle# .cs $arg (insufficient access)."
            return 0
    }

    if { [llength $arg] == 0 } {
        putdcc $idx "    $chan setting:"
        putdcc $idx "        Status : [UndernetCS:GetWordChannelStatus $chan]"
        putdcc $idx "        Level : [UndernetCS:GetLevel $chan]"
        putdcc $idx "        AutoMode : [UndernetCS:GetAutoMode $chan]"
        putdcc $idx "        Suspended : [UndernetCS:IsSuspended $chan]"
        putdcc $idx "        Check Auth when access suspended: Each [UndernetCS:GetSuspendedAuthCheckFrequency $chan] minutes"
        return 1
    }

    # Access si Ok..
    set status ""
    set automode ""
    set check_auth ""
    set check_susp ""
    set level ""
    set ALL_IS_OK 1
    foreach {cmd val} $arg {
    	# check for status key word.
    	if { $cmd == "status" } {
    	    if { $val == "on" } {
    	    	set status 1
    	    } elseif { $val == "off" } {
        		set status 0
    	    } else {
        		set ALL_IS_OK 0
        		break
    	    }
    	    continue
    	}
    	# Check for automode key word
    	if { $cmd == "automode" } {
    	    if { $val == "op" || $val == "voice" || $val == "none" } {
    		    set automode $val
    		    continue
    	    } else {
    		    set ALL_IS_OK 0
    		    break
    	    }
    	}
    	if { $cmd == "check_susp" } {
    	    set check_susp $val
    	    continue
    	}
    	if { $cmd == "level" } {
    	    if { $val > 0 && $val <=500 } {
    	        set level $val
    	    } else {
    	        set ALL_IS_OK 0
    	    }
    	}
    }
    # end of foreach ...
    if { $ALL_IS_OK == 0 } {
    	putdcc $idx "Syntax error.. $cmd $val isn't valid."
    	putcmdlog "#$handle# .cs $arg (Bad arg)."
    	return 0
    }
    set needsave 0
    putdcc $idx "Channel Service:\n"
    # Check for changing level
    if { $level != "" } {
        if { [UndernetCS:GetLevel $chan] != $level } {
            putdcc $idx "\t Access Level for $chan is now $level."
	    UndernetCS:SetNewLevel "$chan" "$level"
            set needsave 1
        } else {
            putdcc $idx "\t Access Level for $chan is ALREADY set to $level."
        }
    }
    # Status report and change.
    if { $status != "" } {
        if { [UndernetCS:IsEnabled $chan] == $status } {
            putdcc $idx "\t Status for $chan is ALREADY : $status"
        } else {
            set needsave 1
            if { $status == 1 } {
            	UndernetCS:SetEnabled "$chan"
            	putdcc $idx "\t Status for $chan is now: Enabled."
            } else {
            	UndernetCS:SetDisabled "$chan"
            	putdcc $idx "\t Status for $chan is now: Disabled."
            }
        }
    }
    # Automode report and change
    if { $automode != "" } {
        set automode [string toupper $automode]
    	if { [UndernetCS:GetAutoMode $chan] != $automode } {
        	UndernetCS:SetAutoMode "$chan" "$automode"
        	putdcc $idx "\t AUTOMODE for $chan is now: $automode"
        	set needsave 1
        } else {
            putdcc $idx "\t AUTOMODE for $chan is ALREADY : $automode"
        }
    }
    # Check when suspended
    if { $check_susp != "" } {
    	if { [UndernetCS:GetSuspendedAuthCheckFrequency $chan] != $check_susp } {
        	UndernetCS:SetSuspendedAuthCheckFrequency "$chan" "$check_susp"
        	putdcc $idx "\t Authentication when access is suspended on $chan is now checked any $check_susp minutes."
        	set needsave 1
        } else {
            putdcc $idx "\t Authentication when access is suspended on $chan is ALREADY checked any $check_susp minutes."
        }
    }
    if { $needsave == 1 } {
	UndernetCS:IO:SaveChannelSetting "$chan"
    }
    return 1
}
### EOF ###