###############################################################################
#                 Undernet Channel Service Management Script                  #
###############################################################################
# UndernetCS-IO.tcl                                                           #
###############################################################################
# Copyright (c) 2001 RICHARD Yann alias Ze <Ze@administrateur.net>            #
###############################################################################

###############################################################################
# This program is distributed under the GNU General Public Licence            #
#  See the LICENCE file for more information.                                 #
###############################################################################


# Need inidb.tcl !!!!!
######################

# UndernetCS:IO:SaveConfigFile 
#   Save Default, Global and Channel setting from 'conf'
#   if there is no arg, Save to config file who are defined
#   in UndernetCS.tcl
#   Return: Nothing.
proc UndernetCS:IO:SaveConfigFile {{conf ""}} {

    global UndernetCS_ConfFile UndernetCS_GlobalItemList
    global UndernetCS_ChannelSetting UndernetCS_ChannelItemList 
    global UndernetCS_X
    global UndernetCS_Prompt 
    
    if { $conf == "" } {
        set conf $UndernetCS_ConfFile
    }
    
    if { [UndernetCS:IO:DebugIsOn] } {
        putlog "Saving All settings in $conf."
    }
    # Saving Default value.
    # There is default value only for channel settings.
    foreach item $UndernetCS_ChannelItemList {
        ini_write "$conf" "DEFAULT" "$item" "$UndernetCS_ChannelSetting(DEFAULT,$item)"
    }
    
    # Saving Global settings
    foreach item $UndernetCS_GlobalItemList {
        ini_write "$conf" "GLOBAL" "$item" "$UndernetCS_X($item)"
    }

    # Saving value for channel...
    foreach chan [channels] {
        set chan [string toupper $chan]
        if { [info exists UndernetCS_ChannelSetting($chan,LEVEL)] == 0 } {
            # This is probably a new channel added since the last control...
            # This proc add setting by default and save them to conf file ..
            UndernetCS:AddChan "$chan"
            continue
        }

        # Now saving settings.
        foreach item $UndernetCS_ChannelSetting  {
            ini_write "$conf" "$chan" "$item" "$UndernetCS_ChannelSetting($chan,$item)"
        }
    }
}

# UndernetCS:IO:LoadConfigFile 
#   Load Default, Global and Channel setting from 'conf'
#   if there is no arg, Save to config file who are defined
#   in UndernetCS.tcl
#   Return: Nothing.
proc UndernetCS:IO:LoadConfigFile {{conf ""}} {
    global UndernetCS_ConfFile UndernetCS_GlobalItemList
    global UndernetCS_ChannelSetting UndernetCS_ChannelItemList 
    global UndernetCS_X UndernetCS_Prompt 
    global UndernetCS_Prompt 
    
    if { $conf == "" } {
        set conf $UndernetCS_ConfFile
    }

    if { [UndernetCS:IO:DebugIsOn] } {
        putlog "$UndernetCS_Prompt Load All settings in $conf."
    }

    # Loading default value...
    foreach item $UndernetCS_ChannelItemList {
        set res [ini_read "$conf" "DEFAULT" "$item"]
        if { $res != "\;" } {
            set UndernetCS_ChannelSetting(DEFAULT,$item) $res
        }
    }

    # Loading Global settings
    foreach item $UndernetCS_GlobalItemList {
        set res [ini_read "$conf" "GLOBAL" "$item"]
        if { $res == "\;" } {
            # Hummm there is no value for this important things...
            # Set status of the script to disabled..
            set UndernetCS_X(STATUS) 0
            putlog "$UndernetCS_Prompt There is no value in config file for $item !!"
            putlog "$UndernetCS_Prompt Script is now disabled."
        } else {
            set UndernetCS_X($item) $res
        }
    }

    # Loading value for channel...
    set newchan ""
    foreach chan [channels] {
        set chan [string toupper $chan]
        putlog "$UndernetCS_Prompt read channel setting for $chan"
        foreach item $UndernetCS_ChannelItemList {
            # Now reading settings.
            set res [ini_read "$conf" "$chan" "$item"] 
            if { $res == "\;" } {
                putlog "$UndernetCS_Prompt $chan : $item vide"
                # This proc create value with default settings and load them 
                # from the config file ... 
                lappend newchan $chan
                UndernetCS:AddChan "$chan"
                break
            } else {
                set UndernetCS_ChannelSetting($chan,$item) $res
            }
        }
    }
    if { $newchan != "" } {
        putlog "$UndernetCS_Prompt New Channel(s) detected: $newchan"
    }
}

# UndernetCS:IO:SaveChannelSetting 
#   Save channel setting for 'channel' to the config file.
#   Return: Nothing.
proc UndernetCS:IO:SaveChannelSetting {channel} {
    global UndernetCS_ConfFile
    global UndernetCS_ChannelSetting UndernetCS_ChannelItemList 
    global UndernetCS_Prompt 
    
    set chan [string toupper $channel]
    set conf $UndernetCS_ConfFile

    if { [UndernetCS:IO:DebugIsOn] } {
        putlog "Saving  All settings in $conf for $chan."
    }

    foreach item $UndernetCS_ChannelItemList {
        ini_write "$conf" "$chan" "$item" "$UndernetCS_ChannelSetting($chan,$item)"
    }
}

proc UndernetCS:IO:SaveGlobalSetting {} {
    global UndernetCS_ConfFile UndernetCS_GlobalItemList
    global UndernetCS_ChannelSetting UndernetCS_ChannelItemList 
    global UndernetCS_X
    global UndernetCS_Prompt 
    
    
    set conf $UndernetCS_ConfFile
    if { [UndernetCS:IO:DebugIsOn] } {
        putlog "Saving  All global settings in $conf."
    }
    
    # Saving Default value.
    # There is default value only for channel settings.
    foreach item $UndernetCS_ChannelItemList {
        ini_write "$conf" "DEFAULT" "$item" "$UndernetCS_ChannelSetting(DEFAULT,$item)"
    }
    
    # Saving Global settings
    foreach item $UndernetCS_GlobalItemList {
        ini_write "$conf" "GLOBAL" "$item" "$UndernetCS_X($item)"
    }
}

proc UndernetCS:IO:RemoveChannelSetting {channel} {
    global UndernetCS_ConfFile UndernetCS_GlobalItemList
    global UndernetCS_ChannelSetting UndernetCS_ChannelItemList 
    global UndernetCS_X
    global UndernetCS_Prompt 

    set conf $UndernetCS_ConfFile

    if { [UndernetCS:IO:DebugIsOn] } {
        putlog "Remove channel settings for $channel in $conf."
    }
    ini_remove $conf [string toupper $channel]
}

# UndernetCS:IO:DebugIsOn
#   Check if debug is enabled for the 'AUTH' section.
#   Return: 1 if enabled.
#           0 if disabled.
proc UndernetCS:IO:DebugIsOn {} {
    global UndernetCS_Debug
    return $UndernetCS_Debug(IO)
}
