###############################################################################
#                 Undernet Channel Service Management Script                  #
###############################################################################
# UndernetCS-Need.tcl                                                         #
###############################################################################
# Copyright (c) 2001 RICHARD Yann alias Ze <Ze@administrateur.net>            #
###############################################################################

###############################################################################
# This program is distributed under the GNU General Public Licence            #
#  See the LICENCE file for more information.                                 #
###############################################################################

# just declare this var ...
set UndernetCS_Need(DEFAULT,OP) 0

bind need - * UndernetCS:Need:EggNeed
proc UndernetCS:Need:EggNeed {channel type} {
    global UndernetCS_Prompt UndernetCS_Need UndernetCS_Nick 

    if { ![UndernetCS:Need:VerifyBasicThings "$channel" "$type"] } {
        return 0
    }

    # Anti internal Need flood... (when X lag for example..)
    # Only 1 need op each 20s..
    set chan [string toupper $channel]
    set type [string toupper $type]
    if { [info exists UndernetCS_Need($chan,$type)] } {
        set UndernetCS_Need($chan,$type) 1
        if { [UndernetCS:Need:DebugIsOn] } {
            putlog "$UndernetCS_Prompt Need $type is ignored for $channel.."
        }
        return 0
    }
    set UndernetCS_Need($chan,$type) 1
    UndernetCS:Need:SetTimer $chan $type

    # Verify if eggdrop isn't already authenticated
    if { [UndernetCS:IsAuth] } {
        if { [UndernetCS:Need:DebugIsOn] } {
            putlog "$UndernetCS_Prompt I'am already authenticated."
        }
        # Eggdrop is so curently authenticated...
        # Verify if suspended ...
        if { [UndernetCS:IsSuspended $channel] } {
            if { [UndernetCS:Need:DebugIsOn] } {
                putlog "$UndernetCS_Prompt My access is suspended on $channel."
            }
            return
        }
        if { $type == "op" } {
            if { ![onchan $UndernetCS_Nick $chan] } {
                if { [UndernetCS:Need:DebugIsOn] } {
                    putlog "$UndernetCS_Prompt X isn't on $chan"
                }
                return 0
            }
	    # Sending op command to X
	    set res [UndernetCS:Cmd:Op $channel]
            return
        } else {
            # Others type can be resolved with invite ...
            set res [UndernetCS:Cmd:Invite $channel]
        }
        return
    }
    # Verify if eggdrop is waiting for a reply to a login request.
    if { [UndernetCS:IsWaitingForReply] } {
        # TODO verify a timeout of the request ...
        if { [UndernetCS:Need:DebugIsOn] } {
            putlog "$UndernetCS_Prompt We are waiting for a reply from X."
        }
        return
    }
    # We aren't authenticated and eggdrop isn't waiting for a reply from X.
    # so we are going to start the auth procedure.
    UndernetCS:Auth:Login2X
    # We don't send now a op cmd, this events is managed when auth succ
    # is receive.
}

proc UndernetCS:Need:VerifyBasicThings {channel type} {
    global UndernetCS_Prompt UndernetCS_Nick
    if { ![UndernetCS:IsActived] } {
        if { [UndernetCS:Need:DebugIsOn] } {
            putlog "$UndernetCS_Prompt Need $type. Script is actualy disabled."
        }
        return 0
    }
    # Testing if eggdrop have an access for this channel.
    if { ![UndernetCS:HaveAccess $channel] } {
        if { [UndernetCS:Need:DebugIsOn] } {
            putlog "$UndernetCS_Prompt Need $type. No access for this channel."
        }
        return 0
    }

    if { ($type == "OP") && ![onchan "$UndernetCS_Nick" "$channel"] } {
        if { [UndernetCS:Need:DebugIsOn] } {
            putlog "$UndernetCS_Prompt Need $type. X are not in $channel."
        }
        return 0
    }
    
    if { [UndernetCS:Need:DebugIsOn] } {
        putlog "$UndernetCS_Prompt Need $type."
    }
    return 1
}


#   Check if debug is enabled for the 'NEED' section.
#   Return: 1 if enabled.
#           0 if disabled.
proc UndernetCS:Need:DebugIsOn {} {
    global UndernetCS_Debug
    return $UndernetCS_Debug(NEED)
}

proc UndernetCS:Need:SetTimer {chan type} {
    global UndernetCS_Prompt
    set chan [UndernetCS:Parse $chan]
    set t [utimer 20 "unset UndernetCS_Need($chan,$type)"]
    putlog "$UndernetCS_Prompt Need:SetTimer --> $t"
}

