###############################################################################
#                 Undernet Channel Service Management Script                  #
###############################################################################
# UndernetCS.tcl                                                              #
###############################################################################
# Copyright (c) 2001 RICHARD Yann alias Ze <Ze@administrateur.net>            #
###############################################################################

###############################################################################
# This program is distributed under the GNU General Public Licence            #
# See the LICENCE file for more information.                                  #
###############################################################################

# Verify compatibility with eggdrop version ...
if {(![info exists numversion]) || ($numversion < 1060000) } {
    putlog "*** Can't load UndernetCS*.tcl -- At least Eggdrop v1.6.x required."
    return 0
}
# Loading configuration.
source scripts/UndernetCS/UndernetCS-Conf.tcl
# Loading inidb.tcl ?
if { $UndernetCS_Must_Load_inidb } {
    if { ![info exists inidb_ver] } {
        source scripts/UndernetCS/inidb.tcl
    }
} else {
    if { ![info exists inidb_ver] } {
        set out "You must turn on the UndernetCS_Must_Load_inidb"
        set out "$out var on UndernetCS-Conf.tcl or loading inidb.tcl" 
        set out "$out before loading UndernetCS.tcl !!!"
        putlog "$out"
        return 0
    }
}

# Current version of the script. !Do not change it!
set UndernetCS_Version "0.3"

# Set script's status to Off
set UndernetCS_X(STATUS) 1
# Default checktime to verify auth status to X (in minutes)
set UndernetCS_X(AUTH_CHK_FREQUENCY) 5


# Prompt before message in eggdrop log
set UndernetCS_Prompt "UndernetCS -->"
# Eggdrop Setting in X database

# Item List
# This is all "index" of the 'UndernetCS_X' array who are saved on
# the config file...
set UndernetCS_GlobalItemList {
    "USERID" "PASS" "STATUS" "AUTH_CHK_FREQUENCY"
}
# All theirs index of the UndernetCS_ChannelSetting's array value
# are saved on config file..
set UndernetCS_ChannelItemList {
    "LEVEL" "STATUS" "AUTOMODE" "SUSPENDED"
    "AUTH_CHK_FREQUENCY_WHEN_SUSP"
}

# Volatile GLOBAL VAR
set UndernetCS_X(NEWPASS) {}
set UndernetCS_X(AUTH_STATUS) 0
###
    # Channel settings
    ###################################
    ## Channel Setting in X database ##
    # Default Level [0..500]
    #   0     = No Access for this channel
    #   1-500 = I have an access for this channel
    set UndernetCS_ChannelSetting(DEFAULT,LEVEL) 0
    # Is the channel is active ? [/0/1]
    #   0 = script disabled for this channel
    #   1 = script enabled for this channel
    set UndernetCS_ChannelSetting(DEFAULT,STATUS) 0
    # Is the access suspended by default  ? [0/1]
    set UndernetCS_ChannelSetting(DEFAULT,SUSPENDED) 0
    # What is the AutoMode by default? [NONE|OP|VOICE]
    set UndernetCS_ChannelSetting(DEFAULT,AUTOMODE) "NONE"
    #################################
    ## Script Setting for Channels ##
    # Default checktime to verify if already suspended (in minutes)
    set UndernetCS_ChannelSetting(DEFAULT,AUTH_CHK_FREQUENCY_WHEN_SUSP) 30


#### DEBUG ####
set UndernetCS_Debug(AUTH) 1    ;# debug for 'Auth' part.
set UndernetCS_Debug(GLOBAL) 0  ;# debug for global features.
set UndernetCS_Debug(NEED) 1    ;# debug for Need part.
set UndernetCS_Debug(IO) 1    ;# debug for IO part.
# UndernetCS:SetAuthStatus 
#   Change the authentication status to 'value' [-1/0/1/Other value]
#       0  : means not curently authenticated
#       1  : means already authenticated
#      -1  : means that the last try return authentication failed...
#     Other: means eggdrop is waiting for a reply from X to
#            a previous auth request. this value is the unix
#            time at the momment where the command was send.
#   Return: Nothing
proc UndernetCS:SetAuthStatus {value} {
    global UndernetCS_X
    set UndernetCS_X(AUTH_STATUS) $value
}

# UndernetCS:IsAuth 
#   Check if eggdrop is authenticated to X.
#   Return: 1 if eggdrop is curently authenticated,
#           0 otherwise.
proc UndernetCS:IsAuth {} {
    global UndernetCS_X
    if { $UndernetCS_X(AUTH_STATUS) == 1 } {
        return 1
    } else { return 0 }
}

# UndernetCS:IsAuthFailed 
#   Check if the last anwser after sending auth command is FAILED.
#   Return: 1 if last try return FAILED.
#           0 otherwise.
proc UndernetCS:IsAuthFailed {} {
    global UndernetCS_X
    if { $UndernetCS_X(AUTH_STATUS) == -1 } {
        return 1
    } else { return 0 }
}

# UndernetCS:IsWaitingForReply 
#   Check if the eggdrop is waiting for a reply to a login request.
#   Return: 0 if eggdrop isn't waiting for a reply to a login request.
#           "the time of the send of the request in unixtime otherwise
proc UndernetCS:IsWaitingForReply {} {
    global UndernetCS_X
    if { $UndernetCS_X(AUTH_STATUS) > 1 } {
        return $UndernetCS_X(AUTH_STATUS)
    } else { return 0 }
}

# UndernetCS:SetPass
#   Just change the internal value of the password of 
#   the access to X. This proc don't send anything to X.
#   Return: Nothing
proc UndernetCS:SetPass {newpass} {
    global UndernetCS_X
    set UndernetCS_X(PASS) $newpass
}

# UndernetCS:IsValidPass 
#   Verify if the parameter will be accepted if it is sending to X
#   as a newpassword.
#   Return: 1 if it is ok.
#           0 otherwise.
proc UndernetCS:IsValidPass {pass} {
    if { [string length $pass] < 6 } {
        return 0
    } else {
        return 1
    }
}

# UndernetCS:GetPass 
#   Return: The password of the eggdrop witch is use
#           for authenticated eggdrop to X.
proc UndernetCS:GetPass {} {
    global UndernetCS_X
    return $UndernetCS_X(PASS)
}

# UndernetCS:SetUserId 
#   Change the UserId of the eggdrop who is used to auth the
#   eggdrop to UnderNet Channel Service.
#   In theory this proc is launch one time if the owner haven't
#   set this var when he have installed the script ...
#   Return: Nothing
proc UndernetCS:SetUserId {newuserid} {
    global UndernetCS_X
    set UndernetCS_X(USERID) $newuserid
}

# UndernetCS:GetUserId
#   Return: The username witch identify the eggdrop in X's database.
proc UndernetCS:GetUserId {} {
    global UndernetCS_X
    return $UndernetCS_X(USERID)
}

# UndernetCS:SetAutoMode
#   Change the internal AutoMode value for "channel".
#   This proc doesn't send anything to X.
#   Return: Nothing
proc UndernetCS:SetAutoMode {channel value} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    set UndernetCS_ChannelSetting($chan,AUTOMODE) [string toupper $value]
}

# UndernetCS:GetAutoMode
#   Return: The value of AUTOMODE for 'channel'
proc UndernetCS:GetAutoMode {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    return $UndernetCS_ChannelSetting($chan,AUTOMODE)
}

# UndernetCS:IsAutoModeOp 
#   Check if AUTOMODE in 'channel' is "OP"
#   Return: 1 if is set to OP,
#           0 otherwise.
proc UndernetCS:IsAutoModeOp {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    if { $UndernetCS_ChannelSetting($chan,AUTOMODE) == "OP" } {
        return 1
    } else {
        return 0
    }
}

# UndernetCS:SetNewLevel 
#   Set the internal value of the access level for 'channel'
#   to 'newlevel'. This proc doesn't send anything to X.
#   Return: Nothing
proc UndernetCS:SetNewLevel {channel newlevel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    if { $newlevel == 0 } {
        # We also disabled this channel
        set UndernetCS_ChannelSetting($chan,STATUS) 0
    }
    set UndernetCS_ChannelSetting($chan,LEVEL) $newlevel
}

# UndernetCS:GetLevel 
#   Return: The access level of eggdrop for 'channel'
proc UndernetCS:GetLevel {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    return $UndernetCS_ChannelSetting($chan,LEVEL)
}

# UndernetCS:SetSuspended 
#   Set SUSPENDED boolean to "value" for the given channel.
#   Return: Nothing
proc UndernetCS:SetSuspended {channel value} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    set UndernetCS_ChannelSetting($chan,SUSPENDED) $value
}

# UndernetCS:IsSuspended 
#   Check if eggdrop's access is suspended for 'channel'
#   Return: 1 if access is suspended,
#           0 if not.
proc UndernetCS:IsSuspended {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    if { $UndernetCS_ChannelSetting($chan,SUSPENDED) == 1 } {
        return 1
    } else {
        return 0
    }
}

# UndernetCS:SetAuthCheckFrequency
#   Set the authentication check frequency to 'value'
#   This value means that each 'value' minuts, the eggdrop check
#   if he is already logged on X. In theory, when the eggdrop check
#   he must be already loggued because the script manage all situation
#   who eggdrop must send an login command to X.
#   Return: Nothing
proc UndernetCS:SetAuthCheckFrequency {value} {
    global UndernetCS_X
    # Unbind and rebind time binding for check
    unbind time - "$UndernetCS_X(AUTH_CHK_FREQUENCY) * * * *" UndernetCS:Auth:AuthTimeChecker
    set UndernetCS_X(AUTH_CHK_FREQUENCY) $value
    bind time - "$UndernetCS_X(AUTH_CHK_FREQUENCY) * * * *" UndernetCS:Auth:AuthTimeChecker
}

# UndernetCS:GetAuthCheckFrequency
#   Return: The value of the authentication check frequency
proc UndernetCS:GetAuthCheckFrequency {} {
    global UndernetCS_X
    return $UndernetCS_X(AUTH_CHK_FREQUENCY)
}

# UndernetCS:SetSuspendedAuthCheckFrequency 
#   Set the frequency of check when the access is suspended on 'channel'
#   to 'value'. This value means that when the access to 'channel' is
#   suspended the eggdrop check each 'value' minutes if he is already
#   suspended. In theory, this situation must not arrived !!
#   But to save the pass-band the script must managed this stupid situation.
#   Return: Nothing
proc UndernetCS:SetSuspendedAuthCheckFrequency {channel value} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    set UndernetCS_ChannelSetting($chan,AUTH_CHK_FREQUENCY_WHEN_SUSP) $value
}

proc UndernetCS:GetSuspendedAuthCheckFrequency {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    return $UndernetCS_ChannelSetting($chan,AUTH_CHK_FREQUENCY_WHEN_SUSP)
}
# UndernetCS:IsActived
#   Check if the script is active not not.
#   Return: 1 if active,
#           0 if inactive.
proc UndernetCS:IsActived {} {
    global UndernetCS_X
    return $UndernetCS_X(STATUS)
}

proc UndernetCS:GetWordScriptStatus {} {
    global UndernetCS_X
    if { $UndernetCS_X(STATUS) } {
        return "Enabled"
    } else {
        return "Disabled"
    }
}


# UndernetCS:SetGlobalStatus 
#   Set Global STATUS of the script to value.
#   Return: Nothing.
proc UndernetCS:SetGlobalStatus {value} {
    global UndernetCS_X
    set UndernetCS_X(STATUS) $value
}


# UndernetCS:HaveAccess
#   Check if eggdrop have an access to X for 'channel'
#   Return: 1 if have an access,
#           0 if don't have an access.
proc UndernetCS:HaveAccess {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    if { $UndernetCS_ChannelSetting($chan,LEVEL) == 0 } {
        return 0
    } else {
        return 1
    }
}

# UndernetCS:IsEnabled 
#   Check if 'channel' is enabled for the script.
#   Return: 1 if enabled,
#           0 if disabled.
proc UndernetCS:IsEnabled {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    if { $UndernetCS_ChannelSetting($chan,STATUS) == 0 } {
        return 0
    } else {
        return 1
    }
}
# UndernetCS:SetEnabled 
#   set to active 'channel' (only if there is a level > 0)
#   Return: 1 if success
#           0 on failed. (level = 0)
proc UndernetCS:SetEnabled {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    if { $UndernetCS_ChannelSetting($chan,LEVEL) == 0 } {
        return 0
    }
    set UndernetCS_ChannelSetting($chan,STATUS) 1
    return 1
}

# UndernetCS:SetDisabled 
#   Unactice 'channel' for the script
#   Return: Nothing.
proc UndernetCS:SetDisabled {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    set UndernetCS_ChannelSetting($chan,STATUS) 0
}

proc UndernetCS:SetChannelStatus {channel value} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    set UndernetCS_ChannelSetting($chan,STATUS) $value
}

proc UndernetCS:GetWordChannelStatus {channel} {
    global UndernetCS_ChannelSetting
    set chan [string toupper $channel]
    if { $UndernetCS_ChannelSetting($chan,STATUS) } {
        return "Enabled"
    } else {
        return "Disabled"
    }
}


# UndernetCS:IsX
#   Return:
#       1: if nick!userhost is the Undernet Channel service Bot
#       0: if nick!userhost isn't Undernet Channel Service Bot.
proc UndernetCS:IsX {nick userhost} {
    global UndernetCS_XMask
    set mask [string tolower "${nick}!${userhost}"]
    if { $mask != $UndernetCS_XMask } {
        return 0
    } else {
        return 1
    }
}

# UndernetCS:AddChan 
#   set channel setting for 'channel' and save them to config file.
#   Return: Nothing.
proc UndernetCS:AddChan {channel} {
    global UndernetCS_ChannelSetting UndernetCS_ChannelItemList 
    set chan [string toupper $channel]
    foreach item $UndernetCS_ChannelItemList {
        set UndernetCS_ChannelSetting($chan,$item) $UndernetCS_ChannelSetting(DEFAULT,$item)
    }
    UndernetCS:IO:SaveChannelSetting $chan
}

# UndernetCS:RemChan 
#   set channel setting for 'channel' and save them to config file.
#   Return: Nothing.
proc UndernetCS:RemChan {channel} {
    global UndernetCS_ChannelSetting UndernetCS_ChannelItemList 
    set chan [string toupper $channel]
    foreach item $UndernetCS_ChannelItemList {
        unset UndernetCS_ChannelSetting($chan,$item)
    }
    UndernetCS:IO:RemoveChannelSetting $chan
}

# UndernetCS:EggdropCanSendCommand2X 
# Return: 1 if eggdrop can send command to X for 'chan'.
#         2 if eggdrop isn't authenticated to X.
#         3 if eggdrop is waiting for a auth reply from X.
#         4 if 'chan' isn't enabled.
#         5 if X isn't on 'chan'.
#         6 if the script is disabled.
proc UndernetCS:EggdropCanSendCommand2X {chan} {
    global UndernetCS_Nick

    if { ![UndernetCS:IsActived] } {
        return 6
    }
    if { ![UndernetCS:IsAuth] } {
        return 2
    }
    if { [UndernetCS:IsWaitingForReply] } {
        return 3
    }
    if { ![UndernetCS:IsEnabled $chan] } {
        return 4
    }
    if { ![onchan "$UndernetCS_Nick" "$chan"] } {
        return 5
    }
    return 1
}

proc UndernetCS:Parse {str} {
    regsub -all {\[} $str {\[} str
    regsub -all {\]} $str {\]} str
    regsub -all {\{} $str {\{} str
    regsub -all {\}} $str {\}} str
    return $str
}

############ SCRIPT INIT ############
# load all UndernetCS*.tcl
source scripts/UndernetCS/UndernetCS-Auth.tcl
source scripts/UndernetCS/UndernetCS-DCC.tcl
source scripts/UndernetCS/UndernetCS-IO.tcl
source scripts/UndernetCS/UndernetCS-IRC.tcl
source scripts/UndernetCS/UndernetCS-Need.tcl
source scripts/UndernetCS/UndernetCS-Cmd.tcl

proc UndernetCS:Init {} {
	global UndernetCS_X UndernetCS_ConfFile UndernetCS_ChannelItemList
	global UndernetCS_GlobalItemList UndernetCS_ChannelSetting

	# Loading config file...
        set active [UndernetCS:IsActived]
        UndernetCS:SetGlobalStatus 0
	if { [file exists $UndernetCS_ConfFile] } {
	    putlog "Reading config file..."
	    putlog "[channels]"
	    UndernetCS:IO:LoadConfigFile
	} else {
	    # This is the first use of this script...
	    putlog "---------------------------------------------------------------------"
	    putlog "              Undernet Channel Service Management Script"
	    putlog "   This program is distributed under the GNU General Public Licence"
	    putlog "---------------------------------------------------------------------"
	    putlog "This is the first use of UndernetCS*.tcl in this bot."
	    putlog "See .cs ; .+cs ; .-cs command for more information to know"
	    putlog "how to configure this script."
	    putlog "Thank you to use this script. If you like this script, please"
	    putlog "Send me a post card (for my collection)... see README file for"
	    putlog "more information... or go to http://mgm.fr.st/"
	    putlog "Copyright (c) 2001 RICHARD Yann. <Ze@administrateur.net>"
	    putlog "---------------------------------------------------------------------"
	    foreach item $UndernetCS_ChannelItemList {
	        ini_write "$UndernetCS_ConfFile" "DEFAULT" "$item" "$UndernetCS_ChannelSetting(DEFAULT,$item)"
	    }
	    # Saving Global settings
	    foreach item $UndernetCS_GlobalItemList {
	        ini_write "$UndernetCS_ConfFile" "GLOBAL" "$item" "$UndernetCS_X($item)"
	    }
	    foreach channel [channels] {
	        set chan [string toupper $channel]
	        UndernetCS:AddChan "$chan"
	    }
	}
        UndernetCS:SetGlobalStatus $active
}

###
set UndernetCS_blah "\"$UndernetCS_X(AUTH_CHK_FREQUENCY) * * * *\""
if [catch "unbind time - $UndernetCS_blah UndernetCS:Auth:AuthTimeChecker" res_catch] {
    putlog "$UndernetCS_Prompt Unbind Error: $res_catch"
}
unset res_catch
unset UndernetCS_blah
bind time - "$UndernetCS_X(AUTH_CHK_FREQUENCY) * * * *" UndernetCS:Auth:AuthTimeChecker
putlog "$UndernetCS_Prompt UndernetCS:Init --> [utimer 3 UndernetCS:Init]"
putlog "UnderNet Channel Service Management Script V$UndernetCS_Version are loaded."
### EOF ###